/* 
 * agent.h -- receiver/sender descriptor data struct.
 *
 * This file is part of
 *
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#ifndef RLC__AGENT_H
#define RLC__AGENT_H

#include "rlc.h"
#include "ui.h"
#include "rlcif.h"
#include "layers.h"
#include "receiver.h"

typedef struct _sxs_t {
    INSTANCE_DES_MEMB

    int is_on;			/* is the session on ? */
    ui16 port;			/* port # (net order) */
    ui32 addrss[MAX_LAYERS];
    int cpi;			/* client prtocol instance descriptor */
    int have_gui;
#ifdef HAVE_UI
    rlc_gui_error_handle_t error_gui;
    rlc_gui_error_handle_t warning_gui;
#endif
    char *geometry;
    ui32 starttime;
    int isreceiver;

    /* Sender Specific */
    int ch;			/* transmitting socket */
    int ttl;

    ui32 rate;			/* tx rate in l_0, actually period in usec. */
    int mpkts, Mpkts;		/* min and Max pktsize (bytes) */

    ui32 pkts_left;		/* packets left to transmit */

    rlc_tx_packet_t tx_packet;	/* client function that provides data */

    ui8	flags;			/* various flags */
#define ADDR_CONS	0x01
    lay_d l;

    /* Receiver Specific */

    int termdelayed;

    ui32 src;			/* src IP address, it bounds to
				 * the first sender it finds */
    int lkypays;		/* our guess on pkt size */
    int lkypaysv;		/* its sort-of-variance */

    rlc_rx_packet_t     rx_packet;
    cong_t *cong;
    int socks[MAX_LAYERS];
    int have_addr_ass;
    ui32 watch_t, watch_last;
    rx_watchdog_t watch_f;
}sxs_t;

int _rlc_start(int pi);
extern void *rlc_inst_mngr;

#endif  /* RLC__AGENT_H */
