/* 
 * congestion.h -- headers for receiver-side congestion control.
 *
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#ifndef RLC__CONGESTION_H
#define RLC__CONGESTION_H

#include <stdio.h>
#include "rlcif.h"
#include "rlc.h"


#define LEAVE_DEAF 2.1		/* XXX */
#define POLL_INTERVAL 0.1	/* XXX */

#define UNSEEN -1

typedef struct _comp_stats_t {
	unsigned int time;		/* us */
	unsigned short tot_count;
	unsigned short clean_count;
	unsigned short lost;
	unsigned short cong_ev;
	unsigned short max_bwt;		/* kBytes/s */
	unsigned char curr_session;
	unsigned char flags;
} comp_stats_t;

/* todo:
 * - possibly all the time values can be converted from double to
 *   ui32
 */

typedef struct _cong_t {
	int pkt_size_;		/* is used for the paket-pair... */
	int n_lays;		/* number of layers */
	double base_period_;	/* used in the leave_timeout */
	double last_0_time;	/* time of the last base layer packet */
	unsigned short last_0_idx;
				/* last index in the base layer */

	ui32 local_id_;		/* this agent id, */
/* added to allow application to bound the number of layers */
	int lay_limit;
	int init_thresh_;	/* initial thershold, defaults = -1,
				    that means set to the layers with
				    max_bwt_/5 (only for the 1st rise) */
#if UNSUPP
	int start_session_;
#endif

	ui32 last_p_;		/* last packet in the current session */
	int curr_session_;	/* upper layer currently joined */
	double leave_timeout;	/* this is for committing a leave */

	double leave_deaf_;	/* this is the time that needs to
				    past before two subsequnt leaves */
	int lost_in_burst;
	double last_arrival;	/* last packet arrival time */
	double next_poll;	/* for poll logging */
	int max_bwt_;		/* max bandwidth in the bottleneck */
 
	/* congestion control ... */
	int thresh_;		/* exp/linear thershold (in layers),
				   only for the 1st rise */
	char flags;		/* status flags... */
	#define	SEEN_UPS	1	/* seen up synchr. point */
	#define	SEEN_BST	2	/* seen burst start */
	#define	LV_PND		4	/* leave pending */
	#define	JN_PND		8	/* join pending */
	#define	LP_UNS		16	/* last packet sn undetermined */
	#define	fset(f, w)	(f)|=(w)
	#define	fclear(f, w)	(f)&=(~(w))
	#define	fisset(f, w)	((f)&(w))

	/* some statistics ... */
	int tot_count_;		/* total numb. of data pkts received */
	int clean_count_;	/* total numb. of data pkts received  without
				    those rec. in left sessions */
	int lost_;		/* total numb. of pkts lost */
	int cong_ev_;		/* introduced here */


	FILE *eventf, *pollf;

	/* this is for conpat form stats. collecting
	 * and delivery... */
#define MAXSTATSBUFSIZE	1024
	char statsbuf[MAXSTATSBUFSIZE];
	int maxstatbufsize;
	int statsbufoffset;
	rx_stats_deliver_f stats_deliver;
	int stats_gid;

 /*
 ** Delay buffer... 
 ** 
 */
	#define MAX_DELAY 5
	rlc_h *heads[MAX_DELAY];
	timest_t rec_times[MAX_DELAY];
	ui32 gidx[MAX_DELAY];
	int this_lay[MAX_DELAY];
	int del_l, del_c, del_in;
} cong_t;

int congestion_setup(int pi, cong_t *inst);
void congestion_end(int pi, cong_t *inst, timest_t endtime);
void congestion_control(int pi, cong_t *inst, rlc_h *p, int paylen,
	int this_lay, ui32 gidx, timest_t r_time);

#endif /* RLC__CONGESTION_H */
