/*
 * init.c -- library initialization routines
 *
 * This file is part of
 * 
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1998 Lorenzo Vicisano
 *     (vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include "rlcif.h"
#include "sender.h"
#include "receiver.h"
#include "error.h"
#include "event.h"

void *rlc_inst_mngr = NULL;
static int rlc_init_done = 0;
void *rlc_events = NULL;

static void rlc_last_stuff (int i)
{
    receiver_last_stuff (i);
    /* if needed we could add something for the sender */
}

int
rlc_internal_init()
{
  int ret, retvalue = 0;

    if (rlc_init_done) return retvalue;

    rlc_init_done = 1;

    if ((ret = _rlc_init_instance (MAX_RLC_INSTANCES, &rlc_inst_mngr))
	< 0) {
	rlc_internal_set_error_ (-1, ret, NULL);
	return ret;
    }

    if ((rlc_events = rlc_createevlist_()) == NULL) {
	return (rlc_internal_set_error_ (-1, NO_EVLIST, NULL));
    }
    if ((ret = set_default_error_h ()) != 0) {
	rlc_internal_set_error_ (-1, NODEFERR_WARNING, NULL);
	retvalue = NODEFERR_WARNING;
    }
    if (signal(SIGINT, rlc_last_stuff) < 0){
	rlc_internal_set_error_ (-1, NOSIGINT_WARNING, strerror (errno));
	retvalue = NOSIGINT_WARNING;
    }

    return retvalue;
}
