/*
 * re.c --- this is a dummy packet multicast receiver.
 *
 * This file is part of
 * 
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <math.h>
#include "rlc.h"

int data_pkt_len = 1024;
void *events;	/* callback attach. and timers */
char data[MAX_PAYLOAD_SIZE];
int datas;
ui32 period = 1;
ui32 daddr;


void
badcmdline(char *arg)
{
    fprintf(stderr,
	"Bad argument %s\nCommand line:\n\t rlcd [options]\n"
	"options:\n"
	"\t-d <M-cast data address/port>\n", arg);
    exit(1);
}

int
get_data(void *p)
{
  int chan = rlc_showselected(events);
  int len, srcaddrlen;
  struct sockaddr_in srcaddr;

    if ((len = recvfrom(chan, data, sizeof(data), 0,
	(struct sockaddr *)&srcaddr, &srcaddrlen))<=0)
	perror("receiver: recvfrom");
    DEB(fprintf(stderr, "got %d bytes...\n", len))
    return 0;
}

int
deal_term(void *p)
{
   char buf[128];
   int re;

    if((re = read(0, buf, sizeof(buf))) <= 0) {
	if (re < 0)
	    perror("read");
	return -1;
    }
    if (*buf == 'j') {
	join_group(datas, daddr);
	DEB(fprintf(stderr, " Join!!\n"))
    } else if (*buf == 'l') {
	leave_group(datas, daddr);
	DEB(fprintf(stderr, " Leave!!\n"))
    }
    return 0;
}


int
main(int argc, char *argv[])
{
  int i;

  char dataaddr[ADDRLEN];
  n16 dport = 0;
  ui8 ttl=TTL;

    extern char *optarg;
    extern int optind;

    /* parse command line... */
    while ((i = getopt(argc,argv,":d:")) != EOF) {
	switch(i) {
	case 'd': /* data addr/port */
	    if(set_link_addr(optarg, dataaddr, ADDRLEN, &dport, "data")<0)
		badcmdline(optarg);
	    break;
	}
    }
    if (dport == 0)
	set_link_addr(DEFAULT_DPORT, dataaddr, ADDRLEN, &dport, "data");
    fprintf(stderr, "using ttl %d \n", ttl);
    argc -= optind;
    argv += optind;

    daddr = inet_addr(dataaddr) ;

    datas = openrsock(daddr, &dport);
    events = rlc_createevlist();

    rlc_insertchan(events, datas, 145, get_data, NULL);
    rlc_insertchan(events, 0, 45, deal_term, NULL);

    while (rlc_getevent(events, 1)!=EV_NO_PENDING);

    exit(0);
}
