/*
 * ui.c --  UI routines.
 *
 * This file is part of
 * 
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1997, 1998 Lorenzo Vicisano
 *     (vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Lorenzo Vicisano and
 *	other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#ifdef HAVE_UI

#include    <stdio.h>
#include    <strings.h>
#include    <tk.h>
#include    <tcl.h>
#if __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#include    "receiver.h"
#include    "agent.h"
#include    "event.h"
#include    "rlcif.h"
#include    "ui.h"
#include    "bitmaps.h"

#define MAX_EVENTS_WIN 10

/*****
 * We expect that the following are defined in the tcl script:
 *
 *  "set_instance_id %d"
 *  "have_an_error %d \"%s\""
 *  "have_an_warning %d \"%s\""
 *
 * for the receiver:
 *  "set_current %d"
 *  "set_last %d"
 *  "set_limit %d"
 *****/

/*****
 * We register the following commands:
 *
 *  "quit %d" (pi)
 *
 * for the receiver:
 *  "limit_notify %d %d" (limit, pi)
 *****/

char *ui_result;

typedef struct _gui_sessd_t {
	Tcl_Interp *interp;
} ui_sessd_t;

static ui_sessd_t ui_sess[MAX_RLC_INSTANCES];

extern char RLC_TLC_SCRPT[];
extern char RLC_RX_TLC_SCRPT[];
extern char RLC_TX_TLC_SCRPT[];

int
APPCallBack	(ClientData data, Tcl_Interp *interp, int argc, char **argv);

typedef	struct {
    	char	    *name;  /* Name of command		*/
	Tcl_CmdProc *proc;  /* Pointer to procedure	*/
	ClientData  data;   /* Client data		*/
} TKCallbacks, *TKCallPtr;

/* Definitions of Tcl callbacks */

static	TKCallbacks ApplCallbacks[] = {
    {"appcall",		APPCallBack,	NULL},
    {NULL,  	    	NULL,	    	NULL} /**** LIST TERMINATOR ****/
};

static int _rlc_gui_upcall (int pi, char *cmd);

/*++++
Executes a tcl command in the gui context...
*/
int
#if __STDC__
rlc_gui_tcl_exec (int pi, char **ret, char const *fmt, ...)
#else
rlc_gui_tcl_exec (pi, ret, fmt, va_alist)
  int pi;
  char **ret;
  char *fmt;
  va_dcl
#endif
{
  va_list ap;
  char cmd[4096];
  int res;
#if __STDC__
    va_start(ap, fmt);
#else
    va_start(ap);
#endif
    vsprintf(cmd, fmt, ap);
    if ((res = _rlc_gui_upcall(pi, cmd)) != TCL_OK)
	rlc_internal_set_error_(pi, TCL_UI_ERROR, NULL);

    if (ret != NULL) {
	if (res == TCL_OK && ui_result != NULL) {
	    if ((*ret = (char *)malloc(strlen(ui_result)+1)) == NULL)
		rlc_internal_set_error_(pi, TCL_UI_ERROR, NULL);
	    else
		strcpy (*ret, ui_result);
	} else {
	    *ret = NULL;
	}
    }

    NDEB(fprintf(stderr, "rlc_gui_tcl_exec : ");)
    NDEB(vfprintf(stderr, fmt, ap);)
    NDEB(fprintf(stderr, "\n res is : %d\n", res);)
    NDEB(if (ret) fprintf(stderr, "\n ret is : %s\n", *ret);)

    va_end(ap);
    return (res);
}

inline void
ui_place (Tcl_Interp *interp, char *geom)
{
  char cmd[512];
    sprintf (cmd, "wm geometry . %s", geom);
    Tcl_Eval (interp, cmd);
}

static int
_rlc_gui_upcall (int pi, char *cmd)
{
    Tcl_Interp  *theInterpreter;
    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    if((theInterpreter = ui_sess[pi].interp) == NULL)
	return (-1);
    ui_result = theInterpreter->result;
    return(Tcl_Eval (theInterpreter, cmd));
}

#if 0 /* this is currently unused */
int
rlc_gui_loadcallback (int pi, char *name, void *proc,
	void *data, void *delproc)
{
    Tcl_Interp  *theInterpreter;
    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    if((theInterpreter = ui_sess[pi].interp) == NULL)
	return (-1);
    if (Tcl_CreateCommand (
	theInterpreter, name, (Tcl_CmdProc *)proc, (ClientData)data,
	    (Tcl_CmdDeleteProc *)delproc) != TCL_OK)
	return(-1);;
    return (0);
}
#endif

int
rlc_gui_loadtcl (int pi, char *code)
{
    Tcl_Interp  *theInterpreter;
    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    if((theInterpreter = ui_sess[pi].interp) == NULL)
	return (-1);
    if (Tcl_VarEval (theInterpreter, code, NULL) != TCL_OK)
	return (-1);
    return (0);
}

void
rlc_gui_delete (int pi)
{
    /* to implement XXX */
}

int
rlc_gui_init	(int pi, int upper_lay, char *appname, char *geometry)
{
    Tk_Window	mainWindow;
    char *windowName, *class;
    TKCallPtr cbPtr = ApplCallbacks;
    Tcl_Interp  *theInterpreter;

    windowName = class = appname;

    /* shouldnt do it only once XXX */
    theInterpreter = Tcl_CreateInterp();
    Tcl_Init(theInterpreter);
    Tk_Init(theInterpreter);
    if ((mainWindow = Tk_MainWindow(theInterpreter) ) == NULL) {
	return (rlc_internal_set_error_(pi, TCL_UI_ERROR, NULL));
    }
    Tk_SetAppName(mainWindow, windowName);
    Tk_SetClass (mainWindow, class);

    if (geometry != NULL)
	ui_place (theInterpreter, geometry);
    if (cbPtr) {
    	while (cbPtr->name != NULL) {
	    if (cbPtr->data == NULL) {
    	    	Tcl_CreateCommand (
	    	    theInterpreter, cbPtr->name,
		    cbPtr->proc, mainWindow, NULL);
	    } else {
    	    	Tcl_CreateCommand (
	    	    theInterpreter, cbPtr->name,
		    cbPtr->proc, cbPtr->data, NULL);
	    }
    	    cbPtr++;
    	}
    }

#if 0
    Tk_DoWhenIdle ((Tcl_IdleProc *)MapWindow, mainWindow);
#endif

    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    ui_sess[pi].interp = theInterpreter;
    rlc_gui_tcl_exec (pi, NULL, "set ProtoInstance %d", pi);

    if (rlc_settimer(UI_RUN, 100000, (CALLBACK)ui_run, NULL, 1) < 0) {
	rlc_gui_delete (pi);
	return (-1);
    }

    if (Tcl_VarEval (theInterpreter, RLC_TLC_SCRPT, NULL) < 0) {
	rlc_gui_delete (pi);
	return (-1);
    }

    return pi;
}

int
rlc_gui_rx_init (int pi)
{
  int result;
    Tcl_Interp  *theInterpreter;
    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    if((theInterpreter = ui_sess[pi].interp) == NULL)
	return (-1);

    Tk_DefineBitmap(theInterpreter, Tk_GetUid("upto"), upto_bits,
	upto_width, upto_height);

    result = Tcl_VarEval (theInterpreter, RLC_RX_TLC_SCRPT, NULL);
    if (result != TCL_OK) {
    	NDEB(fprintf (stderr, "%s\n", theInterpreter->result);)
 	return (-1); 
    }

    return (0);
}

int
rlc_gui_tx_init (int pi)
{
  int result;
    Tcl_Interp  *theInterpreter;
    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    if((theInterpreter = ui_sess[pi].interp) == NULL)
	return (-1);

    result = Tcl_VarEval (theInterpreter, RLC_TX_TLC_SCRPT, NULL);
    if (result != TCL_OK) {
    	NDEB(fprintf (stderr, "%s\n", theInterpreter->result);)
 	return (-1); 
    }

    return (0);
}

void
rlc_gui_error_handle (int pi, char *e)
{
    Tcl_Interp  *theInterpreter;
    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    if((theInterpreter = ui_sess[pi].interp) == NULL)
	return;

    rlc_gui_tcl_exec (pi, NULL, "have_an_error %d \"%s\"", pi, e);
}

void
rlc_gui_warning_handle (int pi, char *e)
{
    Tcl_Interp  *theInterpreter;
    assert(pi>=0 && pi< MAX_RLC_INSTANCES);
    if((theInterpreter = ui_sess[pi].interp) == NULL)
	return;

    DEB(fprintf (stderr, "have_an_warning %d \"%s\"", pi, e);)
    rlc_gui_tcl_exec (pi, NULL, "have_an_warning %d \"%s\"", pi, e);
}

#if 0
static void
MapWindow   	(Tk_Window theWindow)

{
    while (Tk_DoOneEvent (TK_IDLE_EVENTS) != 0) {
    	;
    }
    Tk_MapWindow (theWindow);
}
#endif

/*
 * TCL command procedure that does everything
 *
 * Called after initial parameter are set up and the window
 *   has been mapped
 *
 */

int
ui_run	    	(void *dummy)
{
  int i, ret=1;
  int id;
  int terminated = 1;
    for (id=GET_SESSION_START, id=rlc_get_allinst(id, rlc_inst_mngr);
	id != GET_SESSION_END; id=rlc_get_allinst(id, rlc_inst_mngr)) {
	sxs_t *sxs = (sxs_t *)_rlc_getinstance(id, rlc_inst_mngr);
	if (!sxs->have_gui) continue;
	terminated = 0;
	if (sxs->isreceiver) {
#if 0
	    rlc_rx_info(id, (void *)&info, &ilen);
	    if (info.nlay-1 != ui_sess[id].nlay) {
		ui_last(ui_sess[id].interp, info.nlay-1);
		ui_limit(ui_sess[id].interp, info.nlay-1);
		ui_sess[id].nlay = info.nlay-1;
	    }
	    if (info.clay != ui_sess[id].clay) {
		ui_current(ui_sess[id].interp, info.clay);
		ui_sess[id].clay = info.clay;
	    }
#endif
	}
    }
    for(i=0; ret && i<MAX_EVENTS_WIN && (Tk_GetNumMainWindows() > 0); i++)
	ret = Tcl_DoOneEvent(TCL_DONT_WAIT);
    if (!terminated &&
	rlc_settimer(UI_RUN, 100000, (CALLBACK)ui_run, NULL, 0) < 0 ) {
	rlc_have_error_ (-1, STOP_UI, NULL);
    }
    return(0);
}

int
APPCallBack	(ClientData data, Tcl_Interp *interp, int argc, char **argv)

{
    int  newlimit;
    int pi;

    if (strcmp (argv [1], "limit_notify") == 0) {
	if (argc == 4) {
    	    Tcl_GetInt (interp, argv [2], &newlimit);
    	    Tcl_GetInt (interp, argv [3], &pi);
	    /* **** call here **** */
	    NDEB(fprintf(stderr, "*** have new limit %d\n", newlimit);)
	    if (pi < 0) return TCL_OK;
	    rlc_rx_limit(pi, newlimit);
	} else {
	    Tcl_AppendResult (interp, "Wrong number of args \"",
	    	argv [1], "\"", NULL);
	    return TCL_ERROR;
	}
    } else if (strcmp (argv [1], "quit") == 0) {
    	Tcl_GetInt (interp, argv [2], &pi);
	if (pi < 0) return TCL_OK;
	return (rlc_abort(pi));
    } else {	/* eat and do nothing ! */
	return TCL_ERROR;
#if 0
    	Tcl_AppendResult (interp, "Bad option \"", argv [1],
	    "\": must be dc, mode or source", NULL);
	return TCL_ERROR;
#endif
    }
    return TCL_OK;    
}

#endif /* HAVE_UI */
